package com.almworks.jira.structure.api;

/**
 * <code>StructureError</code> is an enumeration of all possible error types
 * in the Structure layer. Each error is supplemented by numeric code, which
 * may be used to define localizable error messages.
 *
 * @see StructureException
 */
public enum StructureError {
  // === Generic Errors ===

  GENERIC_ERROR(1000),
  JIRA_DATA_ACCESS_EXCEPTION(1001),
  SEARCH_ERROR(1002),

  // === Bad Request ===
  INVALID_JQL(2001),
  INVALID_FILTER_ID(2002),
  INACCESSIBLE_FILTER(2003),

  // === Permissions Related ===
  ISSUE_NOT_EXISTS_OR_NOT_ACCESSIBLE(4001),
  FOREST_CHANGE_PROHIBITED_BY_PARENT_PERMISSIONS(4002),
  PROJECT_NOT_ENABLED_FOR_STRUCTURE(4003),
  ISSUE_NOT_EDITABLE(4004),
  STRUCTURE_NOT_EXISTS_OR_NOT_ACCESSIBLE(4005),
  NOT_ALLOWED_TO_CREATE_STRUCTURE(4006),
  STRUCTURE_ADMIN_DENIED(4007),
  STRUCTURE_EDIT_DENIED(4008),
  JIRA_ADMIN_DENIED(4009),
  STRUCTURE_PLUGIN_ACCESS_DENIED(4010),
  VERSION_NOT_EXISTS_OR_NOT_ACCESSIBLE(4011),

  // === Integrity Related ===
  ISSUE_MISSING_FROM_STRUCTURE(5001),
  INVALID_MOVE(5002)
  ;

  private final int myCode;

  StructureError(int code) {
    myCode = code;
  }

  /**
   * @return error code
   */
  public int getCode() {
    return myCode;
  }

  /**
   * @return true if this error is related to permissions
   */
  public boolean isPermissionError() {
    return myCode / 1000 == 4;
  }


  public String toString() {
    return super.toString() + "[" + myCode + "]";
  }
}
