package com.almworks.jira.structure.api.job;

import com.opensymphony.user.User;
import com.atlassian.jira.util.I18nHelper;

/**
 * <p><code>StructureJobFeedback</code> allows communication between job owner, job itself and the visual
 * components that display the job information.</p>
 *
 * <p>If you know jobId, you can access its feedback using {@link StructureJobManager#getFeedback} method.</p>
 *
 * <p><strong>This feature is under-developed yet.</strong> This interface is likely to change soon
 * as we provide more and better ways to report progress and other information to the user.</p>
 *
 * @author Igor Sereda
 */
public interface StructureJobFeedback {
  /**
   * Adds a button to the feedback screen that brings the user to the specified link.
   *
   * @param buttonText text on the button
   * @param link link address
   * @return this instance
   */
  StructureJobFeedback addButton(String buttonText, String link);

  /**
   * Sets the name of the job.
   *
   * @param key i18n key of the job name
   * @param arguments arguments for creating a text message
   * @return this instance
   */
  StructureJobFeedback setJobName(String key, Object... arguments);

  StructureJobFeedback setI18nFactory(I18nHelper.BeanFactory i18nFactory);

  /**
   * By default, only the owner of the job may see the feedback. Call this method to allow
   * another user to view the feedback as well.
   *
   * @param user the user that is allowed to view the feedback
   * @return this instance
   */
  StructureJobFeedback addManagingUser(User user);

  /**
   * Checks if this feedback is visible to the specified user.
   *
   * @param user the user to check
   * @return true if the user is allowed to see the feedback
   */
  boolean isVisibleTo(User user);
}
