package com.almworks.jira.structure.api.sync;

import com.almworks.integers.LongList;

/**
 * <p>This is a temporary structure that's passed to a synchronizer and tells what's been changed
 * in structure and in JIRA or in the external system.</p>
 *
 * <p>This data is used in the incremental synchronization to limit what issues should
 * be checked and in what direction the changes should be applied.</p>
 *
 * @author Igor Sereda
 */
public class IncrementalSyncData {
  // accepts sorted disjoint sets

  private final LongList myStructureChanged;
  private final LongList myJiraChanged;
  private final LongList myPreviousParents;

  public IncrementalSyncData(LongList structureChangedSorted, LongList jiraChangedSorted, LongList previousParentsSorted) {
    myStructureChanged = structureChangedSorted;
    myJiraChanged = jiraChangedSorted;
    myPreviousParents = previousParentsSorted;
  }

  /**
   * @return issues that were the object of changes in the structure
   */
  public LongList getStructureChangedSorted() {
    return myStructureChanged;
  }

  /**
   * @return issues that were changes in JIRA (edited and so on)
   */
  public LongList getJiraChangedSorted() {
    return myJiraChanged;
  }

  /**
   * @return issues that used to be parent issues of the issues returned by {@link #getStructureChangedSorted()} -
   * it's not possible to otherwise reach those issues now from the affected issues because Forest has changed. Note that
   * these issues may be subject of changes themselves and may be not present in the forest.
   */
  public LongList getPreviousParentsSorted() {
    return myPreviousParents;
  }

  public String toString() {
    return "(jira:" + myJiraChanged + ", structure:" + myStructureChanged + ", p.p.:" + myPreviousParents + ')';
  }
}
