package com.almworks.jira.structure.api.sync;

import com.almworks.jira.structure.util.ResourcedText;
import com.atlassian.jira.plugin.JiraResourcedModuleDescriptor;
import com.atlassian.jira.plugin.OrderableModuleDescriptor;
import com.atlassian.jira.plugin.util.ModuleDescriptorXMLUtils;
import com.atlassian.jira.security.JiraAuthenticationContext;
import com.atlassian.plugin.Plugin;
import com.atlassian.plugin.PluginParseException;
import com.atlassian.plugin.module.ModuleFactory;
import org.dom4j.Element;

import static com.almworks.jira.structure.util.ResourcedText.EMPTY_TEXT;
import static com.almworks.jira.structure.util.ResourcedText.configuredText;

/**
 * <p>Synchronizer module descriptor, used to access the synchronizer configuration
 * set up in the &lt;structure-synchronizer&gt; module.</p>
 *
 * @author Igor Sereda
 */
public class StructureSynchronizerModuleDescriptor
  extends JiraResourcedModuleDescriptor<StructureSynchronizer> implements OrderableModuleDescriptor
{
  private int myOrder;
  private ResourcedText myLabel;
  private ResourcedText myRules;
  private ResourcedText myOutboundDescription;
  private ResourcedText myInboundDescription;

  public StructureSynchronizerModuleDescriptor(JiraAuthenticationContext authenticationContext,
    ModuleFactory moduleFactory)
  {
    super(authenticationContext, moduleFactory);
  }


  public void init(Plugin plugin, Element element) throws PluginParseException {
    super.init(plugin, element);
    myOrder = ModuleDescriptorXMLUtils.getOrder(element);
    myLabel = configuredText(element, "label");
    myRules = configuredText(element, "rules");
    Element outbound = element.element("outbound");
    myOutboundDescription = outbound == null ? EMPTY_TEXT : configuredText(outbound, "resync-description");
    Element inbound = element.element("inbound");
    myInboundDescription = inbound == null ? EMPTY_TEXT : configuredText(inbound, "resync-description");
  }

  public int getOrder() {
    return myOrder;
  }

  public String getLabel() {
    return myLabel.getText(getI18nBean());
  }

  public String getRules() {
    return myRules.getText(getI18nBean());
  }

  public String getOutboundDescription() {
    return myOutboundDescription.getText(getI18nBean());
  }

  public String getInboundDescription() {
    return myInboundDescription.getText(getI18nBean());
  }
}
