package com.almworks.jira.structure.api.sync;

/**
 * <p>This class provides {@link StructureSynchronizer} with additional methods to send feedback to structure
 * manager.</p>
 *
 * @author Igor Sereda
 */
public interface SyncController {
  /**
   * <p>Requests from the structure manager that an incremental sync is required for the specified issue. This method
   * is typically called when the synchronizer receives a notification that an issue has been changed.</p>
   *
   * <p>Shortly after this method is called, an incremental synchronization may run for a number of issues
   * reported as requiring incremental synchronization.</p>
   *
   * <p>The synchronizer must not call this method if the issue has been changed in the structure - only when
   * it has changed in JIRA or externally.</p>
   *
   * @param issue the ID of the issue
   */
  void incrementalSyncRequired(long issue);

  /**
   * This method is used to check whether current thread already runs the synchronization. In that case, the
   * synchronizer should avoid requesting incremental synchronization, because it may result in a cycle (sync
   * required => synchronizer runs, changes something in JIRA => notification is received => sync required).
   *
   * @return true when the current thread currently runs the synchronization this controller is attached to
   */
  boolean isThisSyncRunningInCurrentThread();
}
