package com.almworks.jira.structure.api.sync;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;

/**
 * <p>This interface is used by the synchronizers to store undo information
 * in the synchronization undo log. Synchronizers should use it to allow JIRA admin
 * to roll back changes if needed.</p>
 *
 * @author Igor Sereda
 * @see <a href="http://wiki.almworks.com/display/structure/Structure+Files+Location">Structure Files Location</a> in the documentation
 */
public interface SynchronizerUndoRecorder {
  /**
   * <p>
   *   Add records to undo log. Each record is a single line that is written as-is to the
   *   undo log. The records may be optionally preceded by the description lines,
   *   which are added as comments.
   * </p>
   * <p>
   *   The format of each record is up to the synchronizer. Generally it should contain
   *   all the information to reverse the change and be readable for the admin / script.
   * </p>
   *
   * @param description optional lines with description of the change
   * @param actions a list of records
   */
  void recordUndo(@Nullable List<String> description, @NotNull List<String> actions);
}

